@@ -18,5 +18,4 @@ class Ackley:
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         n = float(len(x))
-        time.sleep(0.1)
-        return -20.0 * np.exp(-0.2*np.sqrt(sum(x**2)/n)) - np.exp(sum(np.cos(2.0*np.pi*x))/n) + 20 + np.exp(1)
+        return -20.0 * np.exp(-0.2*np.sqrt(sum(x**2)/n)) - np.exp(sum(np.cos(2.0*np.pi*x))/n)
