@@ -1,5 +1,6 @@
 import numpy as np
-import time
+
+
 class Keane:
     def __init__(self, dim=10):
         self.xlow = np.zeros(dim)
@@ -18,6 +19,15 @@ class Keane:
         vec[:, 1] = np.sum(x) - 7.5 * self.dim
         return vec
 
+    def deriv_ineq_constraints(self, x):
+        vec = np.zeros((x.shape[0], 2, x.shape[1]))
+        for i in range(x.shape[0]):
+            xx = x[i, :]
+            for j in range(x.shape[1]):
+                vec[i, 0, j] = -np.prod(np.hstack((xx[:j], xx[j+1:])))
+                vec[i, 1, j] = 1.0
+        return vec
+
     # Evaluate the objective function for a single data point
     def objfunction(self, x):
         """Evaluate the Keane function  at x
@@ -25,7 +35,6 @@ class Keane:
         :param x: Data point
         :return: Value at x
         """
-        time.sleep(0.1)
         n = len(x)
         return -abs((sum(np.cos(x)**4)-2 * np.prod(np.cos(x)**2)) /
                     max([1E-10, np.sqrt(np.dot(1+np.arange(n), x**2))]))
