@@ -2,88 +2,20 @@
 .. module:: test_problems
   :synopsis: Test problems for multi-modal and
              box-constrained global optimization
-.. moduleauthor:: David Eriksson <dme65@cornell.edu>
+
+.. moduleauthor:: David Eriksson <dme65@cornell.edu>,
                  David Bindel <bindel@cornell.edu>
 
 :Module: test_problems
-:Author: David Eriksson <dme65@cornell.edu>
-    David Bindel <bindel@cornell.edu>
+:Author: David Eriksson <dme65@cornell.edu>,
+        David Bindel <bindel@cornell.edu>
+
 """
 
 import random
 from time import time
 import numpy as np
-from poap.controller import ProcessWorkerThread
-import importlib
-
-def validate(obj):
-    """Routine for checking that an implementation of an objective function
-    follows the standard. This method checks everything, but can't make
-    sure that the objective function and constraint methods return values
-    of the correct type since this would involve actually evaluating the
-    objective function which isn't feasible when the evaluations are
-    expensive. If some test fails, an exception is raised through assert.
-
-    :param obj: Objective function
-    """
-    assert hasattr(obj, "dim"), \
-        "Problem dimension required"
-    assert hasattr(obj, "xlow"), \
-        "Numpy array of lower bounds required"
-    assert isinstance(obj.xlow, np.ndarray), \
-        "Numpy array of lower bounds required"
-    assert hasattr(obj, "xup"), \
-        "Numpy array of upper bounds required"
-    assert isinstance(obj.xup, np.ndarray), \
-        "Numpy array of upper bounds required"
-    assert hasattr(obj, "integer"), \
-        "Integer variables must be specified"
-    if len(obj.integer) > 0:
-        assert isinstance(obj.integer, np.ndarray), \
-            "Integer variables must be specified"
-    else:
-        assert isinstance(obj.integer, np.ndarray) or \
-            isinstance(obj.integer, list), \
-            "Integer variables must be specified"
-    assert hasattr(obj, "continuous"), \
-        "Continuous variables must be specified"
-    if len(obj.continuous) > 0:
-        assert isinstance(obj.continuous, np.ndarray), \
-            "Continuous variables must be specified"
-    else:
-        assert isinstance(obj.continuous, np.ndarray) or \
-            isinstance(obj.continuous, list), \
-            "Continuous variables must be specified"
-    if not hasattr(obj, "objfunction"):
-        try:
-            module = importlib.import_module(obj.__module__)
-            objfun = getattr(module, "objfunction")
-            if not issubclass(objfun, ProcessWorkerThread):
-                raise AssertionError("Method 'objfunction' is not implemented")
-        except:
-            raise AssertionError("Method 'objfunction' is not implemented")
-
-    # Check for logical errors
-    assert isinstance(obj.dim, int) and obj.dim > 0, \
-        "Problem dimension must be a positive integer."
-    assert (len(obj.xlow) == obj.dim and
-            len(obj.xup) == obj.dim), \
-        "Incorrect size for xlow and xup"
-    assert all(obj.xlow[i] < obj.xup[i] for i in range(obj.dim)), \
-        "Lower bounds must be below upper bounds."
-    if len(obj.integer) > 0:
-        assert np.amax(obj.integer) < obj.dim and np.amin(obj.integer) >= 0, \
-            "Integer variable index can't exceed " \
-            "number of dimensions or be negative"
-    if len(obj.continuous) > 0:
-        assert np.amax(obj.continuous) < obj.dim and \
-               np.amin(obj.continuous) >= 0, \
-               "Continuous variable index can't exceed " \
-               "number of dimensions or be negative"
-    assert len(np.intersect1d(obj.continuous, obj.integer)) == 0, \
-        "A variable can't be both an integer and continuous"
-    assert len(obj.continuous)+len(obj.integer) == obj.dim, \
-        "All variables must be either integer or continuous"
+from pySOT.utils import check_opt_prob
 
 # ========================= 3-dimensional =======================
 
@@ -95,15 +27,25 @@ class Hartman3:
 
     Global optimum: :math:`f(0.114614,0.555649,0.852547)=-3.86278`
 
+    :param dim: Number of dimensions (has to be = 3)
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=3):
         self.xlow = np.zeros(3)
         self.xup = np.ones(3)
@@ -113,7 +55,7 @@ class Hartman3:
         self.min = -3.86278
         self.integer = []
         self.continuous = np.arange(0, 3)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Hartman 3 function  at x
@@ -157,14 +99,23 @@ class Rastrigin:
 
     Global optimum: :math:`f(0,0,...,0)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
     def __init__(self, dim=10):
         self.xlow = -5.12 * np.ones(dim)
@@ -175,14 +126,17 @@ class Rastrigin:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Rastrigin function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         return 10 * self.dim + sum(x**2 - 10 * np.cos(2 * np.pi * x))
@@ -203,15 +157,25 @@ class Ackley:
 
     Global optimum: :math:`f(0,0,...,0)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
+    :type xup: numpy.array
     :ivar info: Problem information:
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -15 * np.ones(dim)
         self.xup = 20 * np.ones(dim)
@@ -221,19 +185,129 @@ class Ackley:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Ackley function  at x
 
+        :param x: Data point
+        :type x: numpy.array
+        :return: Value at x
+        :rtype: float
+        """
+
+        if len(x) != self.dim:
+            raise ValueError('Dimension mismatch')
+        n = float(len(x))
+        return -20.0 * np.exp(-0.2*np.sqrt(np.sum(x**2)/n)) - \
+            np.exp(np.sum(np.cos(2.0*np.pi*x))/n) + 20 + np.exp(1)
+
+
+class Michalewicz:
+    """Michalewicz function
+
+    .. math::
+        f(x_1,\\ldots,x_n) = -\\sum_{i=1}^n \\sin(x_i) \\sin^{20} \\left( \\frac{ix_i^2}{\\pi} \\right)
+
+    subject to
+
+    .. math::
+        0 \\leq x_i \\leq \\pi
+
+    :param dim: Number of dimensions
+    :type dim: int
+
+    :ivar dim: Number of dimensions
+    :type dim: int
+    :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
+    :ivar xup: Upper bound constraints
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
+    :ivar min: Global optimum
+    :type min: float
+    :ivar integer: Integer variables
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
+    """
+    def __init__(self, dim=10):
+        self.xlow = np.zeros(dim)
+        self.xup = np.pi * np.ones(dim)
+        self.dim = dim
+        self.info = str(dim)+"-dimensional Michalewicz function \n" +\
+                             "Global optimum: ??"
+        self.min = np.NaN
+        self.integer = []
+        self.continuous = np.arange(0, dim)
+        check_opt_prob(self)
+
+    def objfunction(self, x):
+        """Evaluate the Michalewicz function  at x
+
+        :param x: Data point
+        :type x: numpy.array
+        :return: Value at x
+        :rtype: float
+        """
+
+        if len(x) != self.dim:
+            raise ValueError('Dimension mismatch')
+        return -np.sum(np.sin(x) * (np.sin(((1+np.arange(self.dim))
+                                            * x**2)/np.pi)) ** 20)
+
+
+class Levy:
+    """Ackley function
+
+    Details: https://www.sfu.ca/~ssurjano/levy.html
+
+    Global optimum: :math:`f(1,1,...,1)=0`
+
+    :param dim: Number of dimensions
+    :type dim: int
+
+    :ivar dim: Number of dimensions
+    :type dim: int
+    :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
+    :ivar xup: Upper bound constraints
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
+    :ivar min: Global optimum
+    :type min: float
+    :ivar integer: Integer variables
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
+    """
+
+    def __init__(self, dim=10):
+        self.xlow = -5 * np.ones(dim)
+        self.xup = 5 * np.ones(dim)
+        self.dim = dim
+        self.min = 0.0
+        self.info = str(dim)+"-dimensional Levy function \n" +\
+                             "Global optimum: ?"
+        self.integer = []
+        self.continuous = np.arange(0, dim)
+        check_opt_prob(self)
+
+    def objfunction(self, x):
+        """Evaluate the Levy function  at x
+
         :param x: Data point
         :return: Value at x
         """
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
-        n = float(len(x))
-        return -20.0 * np.exp(-0.2*np.sqrt(sum(x**2)/n)) - \
-            np.exp(sum(np.cos(2.0*np.pi*x))/n) + 20 + np.exp(1)
+        a = 20
+        b = 0.2
+        d = 50
+        return -a * np.exp(-b * np.sqrt(np.sum(x ** 2)/d)) -\
+            np.exp(np.sum(np.cos(2 * np.pi * x))/d)
 
 
 class Griewank:
@@ -250,15 +324,25 @@ class Griewank:
 
     Global optimum: :math:`f(0,0,...,0)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -512 * np.ones(dim)
         self.xup = 512 * np.ones(dim)
@@ -268,14 +352,17 @@ class Griewank:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Griewank function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         total = 1
@@ -298,15 +385,25 @@ class Rosenbrock:
 
     Global optimum: :math:`f(1,1,...,1)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -2.048 * np.ones(dim)
         self.xup = 2.048 * np.ones(dim)
@@ -316,19 +413,22 @@ class Rosenbrock:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Rosenbrock function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         total = 0
         for i in range(len(x) - 1):
-            total += 100 * (x[i]**2 - x[i+1])**2 + (x[i] - 1)**2
+            total += 100 * (x[i] ** 2 - x[i+1]) ** 2 + (x[i] - 1) ** 2
         return total
 
 
@@ -346,15 +446,25 @@ class Schwefel:
 
     Global optimum: :math:`f(420.968746,420.968746,...,420.968746)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -512 * np.ones(dim)
         self.xup = 512 * np.ones(dim)
@@ -364,14 +474,17 @@ class Schwefel:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Schwefel function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         return 418.9829 * self.dim - \
@@ -391,15 +504,25 @@ class Sphere:
 
     Global optimum: :math:`f(0,0,...,0)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -5.12 * np.ones(dim)
         self.xup = 5.12 * np.ones(dim)
@@ -409,17 +532,20 @@ class Sphere:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Sphere function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
-        return sum(x**2)
+        return np.sum(x ** 2)
 
 
 class Exponential:
@@ -435,15 +561,25 @@ class Exponential:
 
     Global optimum: :math:`f(0,0,...,0)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -5.12 * np.ones(dim)
         self.xup = 5.12 * np.ones(dim)
@@ -453,14 +589,17 @@ class Exponential:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Exponential function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         total = 0
@@ -484,15 +623,25 @@ class StyblinskiTang:
     Global optimum: :math:`f(-2.903534,-2.903534,...,-2.903534)=\
     -39.16599 \\cdot n`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -5 * np.ones(dim)
         self.xup = 5 * np.ones(dim)
@@ -503,17 +652,20 @@ class StyblinskiTang:
         self.min = -39.16599*dim
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the StyblinskiTang function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
-        return 0.5*sum(x**4 - 16*x**2 + 5*x)
+        return 0.5*np.sum(x ** 4 - 16 * x ** 2 + 5 * x)
 
 
 class Quartic:
@@ -529,15 +681,25 @@ class Quartic:
 
     Global optimum: :math:`f(0,0,...,0)=0+noise`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -1.28 * np.ones(dim)
         self.xup = 1.28 * np.ones(dim)
@@ -549,14 +711,17 @@ class Quartic:
         self.continuous = np.arange(0, dim)
         self.prng = random.Random()
         self.prng.seed(time())
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Quartic function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         total = 0.0
@@ -580,15 +745,25 @@ class Whitley:
 
     Global optimum: :math:`f(1,1,...,1)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -10.24 * np.ones(dim)
         self.xup = 10.24 * np.ones(dim)
@@ -598,14 +773,17 @@ class Whitley:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the Whitley function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         total = 0
@@ -635,15 +813,25 @@ class SchafferF7:
 
     Global optimum: :math:`f(0,0,...,0)=0`
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = -100 * np.ones(dim)
         self.xup = 100 * np.ones(dim)
@@ -653,14 +841,17 @@ class SchafferF7:
         self.min = 0
         self.integer = []
         self.continuous = np.arange(0, dim)
-        validate(self)
+        check_opt_prob(self)
 
     def objfunction(self, x):
         """Evaluate the SchafferF7 function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         total = 0
@@ -683,7 +874,7 @@ class Keane:
     subject to
 
     .. math::
-        -100 \\leq x_i \\leq 100
+        0 \\leq x_i \\leq 5
 
     .. math::
         0.75 - \\prod_{j=1}^n x_j < 0
@@ -694,15 +885,25 @@ class Keane:
 
     Global optimum: -0.835 for large n
 
+    :param dim: Number of dimensions
+    :type dim: int
+
     :ivar dim: Number of dimensions
+    :type dim: int
     :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
     :ivar xup: Upper bound constraints
-    :ivar info: Problem information:
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
     :ivar min: Global optimum
+    :type min: float
     :ivar integer: Integer variables
-    :ivar continuous: Continuous variables:
-    :ivar constraints: Whether there are non-bound constraints
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
+
     def __init__(self, dim=10):
         self.xlow = np.zeros(dim)
         self.xup = 5 * np.ones(dim)
@@ -712,22 +913,48 @@ class Keane:
         self.continuous = np.arange(0, dim)
         self.info = str(dim)+"-dimensional Keane bump function \n" +\
                              "Global optimum: -0.835 for large n"
-        validate(self)
+        check_opt_prob(self)
 
-    # Return a list with the constraint function at the given point
     def eval_ineq_constraints(self, x):
+        """Evaluate the Keane inequality constraints at x
+
+        :param x: Data points, of size npts x dim
+        :type x: numpy.array
+        :return: Value at the constraints, of size npts x nconstraints
+        :rtype: float
+        """
+
         vec = np.zeros((x.shape[0], 2))
         vec[:, 0] = 0.75 - np.prod(x)
         vec[:, 1] = np.sum(x) - 7.5 * self.dim
         return vec
 
-    # Evaluate the objective function for a single data point
+    def deriv_ineq_constraints(self, x):
+        """Evaluate the derivative of the Keane inequality constraints at x
+
+        :param x: Data points, of size npts x dim
+        :type x: numpy.array
+        :return: Derivative at the constraints, of size npts x nconstraints x ndims
+        :rtype: float
+        """
+
+        vec = np.zeros((x.shape[0], 2, x.shape[1]))
+        for i in range(x.shape[0]):
+            xx = x[i, :]
+            for j in range(x.shape[1]):
+                vec[i, 0, j] = -np.prod(np.hstack((xx[:j], xx[j+1:])))
+                vec[i, 1, j] = 1.0
+        return vec
+
     def objfunction(self, x):
-        """Evaluate the Keane function  at x
+        """Evaluate the Keane function at a point x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
                 raise ValueError('Dimension mismatch')
         n = len(x)
@@ -742,8 +969,27 @@ class LinearMI:
     are continuous.
 
     Global optimum: :math:`f(1,0,0,0,0) = -1`
+
+    :param dim: Number of dimensions (has to be 5)
+    :type dim: int
+
+    :ivar dim: Number of dimensions
+    :type dim: int
+    :ivar xlow: Lower bound constraints
+    :type xlow: numpy.array
+    :ivar xup: Upper bound constraints
+    :type xup: numpy.array
+    :ivar info: Problem information
+    :type info: string
+    :ivar min: Global optimum
+    :type min: float
+    :ivar integer: Integer variables
+    :type integer: numpy.array
+    :ivar continuous: Continuous variables
+    :type continuous: numpy.array
     """
-    def __init__(self):
+
+    def __init__(self, dim=5):
         self.xlow = np.zeros(5)
         self.xup = np.array([10, 10, 10, 1, 1])
         self.dim = 5
@@ -753,9 +999,17 @@ class LinearMI:
         self.info = str(self.dim)+"-dimensional Linear MI \n" +\
                                   "Global optimum: f(1,0,0,0,0) = -1\n" +\
                                   str(len(self.integer)) + " integer variables"
-        validate(self)
+        check_opt_prob(self)
 
     def eval_ineq_constraints(self, x):
+        """Evaluate the LinearMI inequality constraints at x
+
+        :param x: Data points, of size npts x dim
+        :type x: numpy.array
+        :return: Value at the constraints, of size npts x nconstraints
+        :rtype: float
+        """
+
         vec = np.zeros((x.shape[0], 3))
         vec[:, 0] = x[:, 0] + x[:, 2] - 1.6
         vec[:, 1] = 1.333 * x[:, 1] + x[:, 3] - 3
@@ -766,8 +1020,11 @@ class LinearMI:
         """Evaluate the LinearMI function  at x
 
         :param x: Data point
+        :type x: numpy.array
         :return: Value at x
+        :rtype: float
         """
+
         if len(x) != self.dim:
             raise ValueError('Dimension mismatch')
         return - x[0] + 3 * x[1] + 1.5 * x[2] + 2 * x[3] - 0.5 * x[4]
@@ -864,3 +1121,11 @@ if __name__ == "__main__":
           str(fun.objfunction(np.array([1, 1, 1, 1, 1]))))
     print("Continuous variables: " + str(fun.continuous))
     print("Integer variables: " + str(fun.integer))
+
+    print("\n========================= Michalewicz =======================")
+    fun = Michalewicz(dim=2)
+    print(fun.info)
+    print("Michalewicz(2.20, 1.57) = " +
+          str(fun.objfunction(np.array([2.20, 1.57]))))
+    print("Continuous variables: " + str(fun.continuous))
+    print("Integer variables: " + str(fun.integer))
